# Compare the efficiency with different cores
library(s2dv)
library(multiApply)
library(rbenchmark)

#------------------------------------------------
#CONCLUSION
# The following result comes from the calculation of 1.5Gb data averaged over 2 dimensions.
# The result shows that apply() is still faster than Apply()+4cores.
# Can we conclude that regarding mean(), apply is better than Apply?

            test replications elapsed relative user.self sys.self
1          apply            1  90.847    1.000    90.082    0.568
2  Apply, 1 core            1 389.495    4.287   388.612    0.052
3 Apply, 4 cores            1 100.110    1.102     0.208    0.152


#-------------------------------------------------

# Mean along 1 dim
            data <- array(rnorm(1:1000),
                          dim = c(dat = 1, time = 12, member = 50, lon = 30, lat = 50))
            dims <- 2

benchmark("apply" = {
            pos <- (1:length(dim(data)))[-dims]
            res <- apply(data, pos, mean, na.rm = T)
          },

          "Apply, 1 core" = {
            res <- Apply(list(data), target_dims = list(dims), fun = mean,
                          na.rm = T, ncores = 1)$output1
          },

          "Apply, 4 cores" = {
            res <- Apply(list(data), target_dims = list(dims), fun = mean,
                          na.rm = T, ncores = 4)$output1
          },
          replications = 5,
          columns = c("test", "replications", "elapsed",
                      "relative", "user.self", "sys.self"))

            test replications  elapsed relative user.self sys.self
1          apply           10  158.860    1.000   158.190    0.660
2  Apply, 1 core           10 1048.982    6.603  1048.190    0.668
3 Apply, 4 cores           10  294.800    1.856     3.037    1.368

            test replications elapsed relative user.self sys.self
1          apply            5   1.371    1.000     1.367    0.004
2  Apply, 1 core            5  12.228    8.919    12.228    0.000
3 Apply, 4 cores            5   3.699    2.698     0.059    0.184

# Mean along two dims
            data <- array(rnorm(1:1000),
                          dim = c(dat = 1, time = 12, member = 50, lon = 30, lat = 50))
            dims <- c(1, 2)
benchmark("apply" = {
            pos <- (1:length(dim(data)))[-dims]
            res <- apply(data, pos, mean, na.rm = T)
          },

          "Apply, 1 core" = {
            res <- Apply(list(data), target_dims = list(dims), fun = mean,
                          na.rm = T, ncores = 1)$output1

          },
          
          "Apply, 4 cores" = {
            res <- Apply(list(data), target_dims = list(dims), fun = mean,
                          na.rm = T, ncores = 4)$output1

          },
          replications = 5,
          columns = c("test", "replications", "elapsed",
                      "relative", "user.self", "sys.self"))

            test replications elapsed relative user.self sys.self
1          apply           10   4.306    1.000     4.292    0.012
2  Apply, 1 core           10  22.892    5.316    22.811    0.068
3 Apply, 4 cores           10   6.380    1.482     0.085    0.264

            test replications elapsed relative user.self sys.self
1          apply            5   2.030    1.000     2.029     0.00
2  Apply, 1 core            5  11.128    5.482    11.128     0.00
3 Apply, 4 cores            5   3.415    1.682     0.023     0.18


# Mean along 4 dims
            dims <- c(1, 2, 3, 4)
benchmark("apply" = {
            pos <- (1:length(dim(data)))[-dims]
            res <- apply(data, pos, mean, na.rm = T)
          },

          "Apply, 1 core" = {
            res <- Apply(list(data), target_dims = list(dims), fun = mean,
                          na.rm = T, ncores = 1)$output1

          },

          "Apply, 4 cores" = {
            res <- Apply(list(data), target_dims = list(dims), fun = mean,
                          na.rm = T, ncores = 4)$output1

          },
          replications = 5,
          columns = c("test", "replications", "elapsed",
                      "relative", "user.self", "sys.self"))

            test replications elapsed relative user.self sys.self
1          apply           10   0.171    1.000     0.163    0.008
2  Apply, 1 core           10   0.206    1.205     0.206    0.000
3 Apply, 4 cores           10   0.424    2.480     0.054    0.264

            test replications elapsed relative user.self sys.self
1          apply            5   0.078    1.000     0.079    0.000
2  Apply, 1 core            5   0.099    1.269     0.098    0.000
3 Apply, 4 cores            5   0.241    3.090     0.041    0.128


#/////////////////// Memory profiling ////////////////////
  data <- array(rnorm(1:1000),
                dim = c(dat = 1, time = 12, member = 50, lon = 360, lat = 180))
  dims <- c(1, 2)

  # apply
  Rprof(memory.profiling = T)
  pos <- (1:length(dim(data)))[-dims]
  res <- apply(data, pos, mean, na.rm = T)
  Rprof(NULL)
  summaryRprof(memory = 'both')

  # Apply, 1 core
  Rprof(interval = 0.01, memory.profiling = T)
  res <- Apply(list(data), target_dims = list(dims), fun = mean,
                na.rm = T, ncores = 1)$output1
  Rprof(NULL)
  summaryRprof(memory = 'both')

  # Apply, 4 cores
  Rprof(memory.profiling = T)
  res <- Apply(list(data), target_dims = list(dims), fun = mean,
                na.rm = T, ncores = 4)$output1
  Rprof(NULL)
  summaryRprof(memory = 'both')

# Result: with 4 cores, the time is the shortest. 1 core and apply have similar time.
#         As for memory, 4 cores consumes "zero", 1 core is 1146.6 and apply is 1413.8.
   


 
#/////////////// peakRAM /////////////////
peakRAM(
  pos <- (1:length(dim(data)))[-dims],
  res <- apply(data, pos, mean, na.rm = T)
)
                      Function_Call Elapsed_Time_sec Total_RAM_Used_MiB
1 pos<-(1:length(dim(data)))[-dims]            0.000                  0
2 res<-apply(data,pos,mean,na.rm=T)           23.836                  0

  Peak_RAM_Used_MiB
1               0.0
2             423.7

peakRAM(
  res <- Apply(list(data), target_dims = list(dims), fun = mean,
                na.rm = T, ncores = 1)$output1
)
                                                                    Function_Call
1 res<-Apply(list(data),target_dims=list(dims),fun=mean,na.rm=T,ncores=1)$output1
  Elapsed_Time_sec Total_RAM_Used_MiB Peak_RAM_Used_MiB
1           97.206                  0             116.5

peakRAM(
  res <- Apply(list(data), target_dims = list(dims), fun = mean,
                na.rm = T, ncores = 4)$output1
)
                                                                    Function_Call
1 res<-Apply(list(data),target_dims=list(dims),fun=mean,na.rm=T,ncores=4)$output1
  Elapsed_Time_sec Total_RAM_Used_MiB Peak_RAM_Used_MiB
1           26.429               24.1               136

