#///////////////////////////////////////////////////////////////////////
#NOTE: This script tests the performance difference between s2dv::Season,
#      s2dverification::Season, and the corresponding compiled ones.
#      Also, with/out input checks is tested. For s2dv::Season, Apply() is 
#      replaced by apply() to see the improvement.
#///////////////////////////////////////////////////////////////////////
#CONCLUSION: 1. input checks does not impact much in this case.
#            2. Compilation does not help improve Season() efficiency. If in a script,
#               compile .Season() then use it in Season() many times is helpful (since
#               compilation is only required once at the beginning). However, if the
#               compilation is in a function, every time the function is called, .Season()
#               needs to be compiled. In total, the time of 'compile + Season()' is similar
#               to the original s2dv::Season.
#            3. apply() is near 3 times faster than Apply() in this case.
#            4. s2dverfication::Season is much faster than others, because Season() is a 
#               simple function. s2dv uses either apply() or Apply(), which takes more time
#               when the operation is simple.
#///////////////////////////////////////////////////////////////////////

library(s2dv)
library(rbenchmark)
library(compiler)
library(multiApply)

#__________ Use compiled function .Season2 in Apply() ______________
.Season2 <- cmpfun(s2dv:::.Season)
Season_outside <- function(data, time_dim = 'ftime', monini, moninf, monsup, 
                   method = mean, na.rm = TRUE, ncores = NULL) {

  # Check inputs 
  ## data
  if (is.null(data)) {
    stop("Parameter 'data' cannot be NULL.")
  }
  if (!is.numeric(data)) {
    stop("Parameter 'data' must be a numeric array.")
  }
  if (is.null(dim(data))) {  #is vector
    dim(data) <- c(length(data))
    names(dim(data)) <- time_dim
  }
  if(any(is.null(names(dim(data))))| any(nchar(names(dim(data))) == 0)) {
    stop("Parameter 'data' must have dimension names.")
  }
  ## time_dim
  if (!is.character(time_dim) | length(time_dim) > 1) {
    stop("Parameter 'time_dim' must be a character string.")
  }
  if (!time_dim %in% names(dim(data))) {
    stop("Parameter 'time_dim' is not found in 'data' dimension.")
  }
  ## monini
  if (!is.numeric(monini)) {
    stop("Parameter 'monini' must be a positive integer between 1 and 12.")
  } else {
    if (monini %% 1 != 0 | monini < 1 | monini > 12 | length(monini) > 1) {
      stop("Parameter 'monini' must be a positive integer between 1 and 12.")
    }
  }
  ## moninf
  if (!is.numeric(moninf)) {
    stop("Parameter 'moninf' must be a positive integer between 1 and 12.")
  } else {
    if (moninf %% 1 != 0 | moninf < 1 | moninf > 12 | length(moninf) > 1) {
      stop("Parameter 'moninf' must be a positive integer between 1 and 12.")
    }
  }
  ## monsup
  if (!is.numeric(monsup)) {
    stop("Parameter 'monsup' must be a positive integer between 1 and 12.")
  } else {
    if (monsup %% 1 != 0 | monsup < 1 | monsup > 12 | length(monsup) > 1) {
      stop("Parameter 'monsup' must be a positive integer between 1 and 12.")
    }
  }
  ## method
  if (!is.function(method)) {
    stop("Parameter 'method' should be an existing R function, e.g., mean or sum.")
  }
  ## na.rm
  if (!is.logical(na.rm) | length(na.rm) > 1) {
    stop("Parameter 'na.rm' must be one logical value.")
  }
  ## ncores
  if (!is.null(ncores)) {
    if (!is.numeric(ncores) | ncores %% 1 != 0 | ncores < 0 |
      length(ncores) > 1) {
      stop("Parameter 'ncores' must be a positive integer.")
    }
  } 

  ###############################
  # Calculate Season

    # Correction need if monini is not January: 
    moninf <- moninf - monini + 1
    monsup <- monsup - monini + 1
    moninf <- ifelse(moninf <= 0, moninf + 12, moninf)
    monsup <- ifelse(monsup <= 0, monsup + 12, monsup)

    while (monsup < moninf) {
        monsup <- monsup + 12
    }

    res <- Apply(list(data), 
                 target_dims = time_dim, 
                 output_dims = time_dim,
                 fun = .Season2, 
                 monini = monini, moninf = moninf, monsup = monsup,
                 method = method, na.rm = na.rm, ncores = ncores)$output1

    return(res)
}

#_________ Change the function in Apply() to cmpfun(s2dv:::.Season) _____________
Season_inside <- function(data, time_dim = 'ftime', monini, moninf, monsup,
                   method = mean, na.rm = TRUE, ncores = NULL) {

  # Check inputs 
  ## data
  if (is.null(data)) {
    stop("Parameter 'data' cannot be NULL.")
  }
  if (!is.numeric(data)) {
    stop("Parameter 'data' must be a numeric array.")
  }
  if (is.null(dim(data))) {  #is vector
    dim(data) <- c(length(data))
    names(dim(data)) <- time_dim
  }
  if(any(is.null(names(dim(data))))| any(nchar(names(dim(data))) == 0)) {
    stop("Parameter 'data' must have dimension names.")
  }
  ## time_dim
  if (!is.character(time_dim) | length(time_dim) > 1) {
    stop("Parameter 'time_dim' must be a character string.")
  }
  if (!time_dim %in% names(dim(data))) {
    stop("Parameter 'time_dim' is not found in 'data' dimension.")
  }
  ## monini
  if (!is.numeric(monini)) {
    stop("Parameter 'monini' must be a positive integer between 1 and 12.")
  } else {
    if (monini %% 1 != 0 | monini < 1 | monini > 12 | length(monini) > 1) {
      stop("Parameter 'monini' must be a positive integer between 1 and 12.")
    }
  }
  ## moninf
  if (!is.numeric(moninf)) {
    stop("Parameter 'moninf' must be a positive integer between 1 and 12.")
  } else {
    if (moninf %% 1 != 0 | moninf < 1 | moninf > 12 | length(moninf) > 1) {
      stop("Parameter 'moninf' must be a positive integer between 1 and 12.")
    }
  }
  ## monsup
  if (!is.numeric(monsup)) {
    stop("Parameter 'monsup' must be a positive integer between 1 and 12.")
  } else {
    if (monsup %% 1 != 0 | monsup < 1 | monsup > 12 | length(monsup) > 1) {
      stop("Parameter 'monsup' must be a positive integer between 1 and 12.")
    }
  }
  ## method
  if (!is.function(method)) {
    stop("Parameter 'method' should be an existing R function, e.g., mean or sum.")
  }
## na.rm
  if (!is.logical(na.rm) | length(na.rm) > 1) {
    stop("Parameter 'na.rm' must be one logical value.")
  }
  ## ncores
  if (!is.null(ncores)) {
    if (!is.numeric(ncores) | ncores %% 1 != 0 | ncores < 0 |
      length(ncores) > 1) {
      stop("Parameter 'ncores' must be a positive integer.")
    }
  }

  ###############################
  # Calculate Season

    # Correction need if monini is not January: 
    moninf <- moninf - monini + 1
    monsup <- monsup - monini + 1
    moninf <- ifelse(moninf <= 0, moninf + 12, moninf)
    monsup <- ifelse(monsup <= 0, monsup + 12, monsup)

    while (monsup < moninf) {
        monsup <- monsup + 12
    }

    res <- Apply(list(data),
                 target_dims = time_dim,
                 output_dims = time_dim,
                 fun = cmpfun(s2dv:::.Season),
                 monini = monini, moninf = moninf, monsup = monsup,
                 method = method, na.rm = na.rm, ncores = ncores)$output1

    return(res)
}

#________ Compile the main functions _________
Comp_Season <- cmpfun(s2dv::Season)
Comp_Season_s2dverification <- cmpfun(s2dverification::Season)

#_________________ Test 1 ___________________

#enableJIT(0)
data <- CSTools::lonlat_data$exp$data

benchmark(
#          "s2dverification::Season" = {
#            res_Regression <- s2dverification::Season(data, monini = 11, moninf = 12,
#                                                      monsup = 1)
#          },
#          "Comp_s2dverification::Season" = {
#            res_Season <- Comp_Season_s2dverification(data, monini = 11,
#                                                moninf = 12, monsup = 1)
#          },
          "s2dv::Season" = {
            res_Season <- s2dv::Season(data, monini = 11, moninf = 12, monsup = 1)
          },
          "s2dv:Season_comp_outside" = {
            .Season2 <- cmpfun(s2dv:::.Season)
            res_Season <- Season_outside(data, monini = 11, moninf = 12, monsup = 1)
          },
          "s2dv::Season_comp_inside" = {
            res_Season <- Season_inside(data, monini = 11, moninf = 12, monsup = 1)
          },

          replications = 10,
          columns = c("test", "replications", "elapsed",
                      "relative", "user.self", "sys.self"))


                      test replications elapsed relative user.self sys.self
1             s2dv::Season           10 106.863    1.012   106.857        0
3 s2dv::Season_comp_inside           10 106.253    1.006   106.247        0
2 s2dv:Season_comp_outside           10 105.593    1.000   105.588        0

#----------------This is wrong!!! outside one compiling time is not counted in benchmark!!!--------
#                      test replications elapsed relative user.self sys.self
#1             s2dv::Season           10 110.777   14.473   110.768    0.000
#3 s2dv::Season_comp_inside           10 109.081   14.252   109.071    0.003
#2 s2dv:Season_comp_outside           10   7.654    1.000     7.654    0.000



#____________ Test 2 __________________
# Test ncores. 
benchmark(
          "s2dverification::Season" = {
            res_Regression <- s2dverification::Season(data, monini = 11, moninf = 12,
                                                      monsup = 1)
          },
          "s2dv::Season_1" = {
            res_Season <- s2dv::Season(data, monini = 11, moninf = 12, monsup = 1)
          },
          "s2dv::Season_2" = {
            res_Season <- s2dv::Season(data, monini = 11, moninf = 12, monsup = 1, ncores = 2)
          },

          replications = 10,
          columns = c("test", "replications", "elapsed",
                      "relative", "user.self", "sys.self"))

#                     test replications elapsed relative user.self sys.self
#2          s2dv::Season_1           10 111.519  648.366   111.512    0.000
#3          s2dv::Season_2           10  56.034  325.779     0.123    0.080
#1 s2dverification::Season           10   0.172    1.000     0.168    0.004

#//////////////////////////////////////////////////////////////////
#//////////////////////////////////////////////////////////////////

#_________ s2dv::Season with apply instead of Apply ______________
Season_apply <- function(data, time_dim = 'ftime', monini, moninf, monsup, 
                   method = mean, na.rm = TRUE, ncores = NULL) {

  # Check inputs 
  ## data
  if (is.null(data)) {
    stop("Parameter 'data' cannot be NULL.")
  }
  if (!is.numeric(data)) {
    stop("Parameter 'data' must be a numeric array.")
  }
  if (is.null(dim(data))) {  #is vector
    dim(data) <- c(length(data))
    names(dim(data)) <- time_dim
  }
  if(any(is.null(names(dim(data))))| any(nchar(names(dim(data))) == 0)) {
    stop("Parameter 'data' must have dimension names.")
  }
  ## time_dim
  if (!is.character(time_dim) | length(time_dim) > 1) {
    stop("Parameter 'time_dim' must be a character string.")
  }
  if (!time_dim %in% names(dim(data))) {
    stop("Parameter 'time_dim' is not found in 'data' dimension.")
  }
  ## monini
  if (!is.numeric(monini)) {
    stop("Parameter 'monini' must be a positive integer between 1 and 12.")
  } else {
    if (monini %% 1 != 0 | monini < 1 | monini > 12 | length(monini) > 1) {
      stop("Parameter 'monini' must be a positive integer between 1 and 12.")
    }
  }
  ## moninf
  if (!is.numeric(moninf)) {
    stop("Parameter 'moninf' must be a positive integer between 1 and 12.")
  } else {
    if (moninf %% 1 != 0 | moninf < 1 | moninf > 12 | length(moninf) > 1) {
      stop("Parameter 'moninf' must be a positive integer between 1 and 12.")
    }
  }
  ## monsup
  if (!is.numeric(monsup)) {
    stop("Parameter 'monsup' must be a positive integer between 1 and 12.")
  } else {
    if (monsup %% 1 != 0 | monsup < 1 | monsup > 12 | length(monsup) > 1) {
      stop("Parameter 'monsup' must be a positive integer between 1 and 12.")
    }
  }
  ## method
  if (!is.function(method)) {
    stop("Parameter 'method' should be an existing R function, e.g., mean or sum.")
  }
  ## na.rm
  if (!is.logical(na.rm) | length(na.rm) > 1) {
    stop("Parameter 'na.rm' must be one logical value.")
  }
  ## ncores
  if (!is.null(ncores)) {
    if (!is.numeric(ncores) | ncores %% 1 != 0 | ncores < 0 |
      length(ncores) > 1) {
      stop("Parameter 'ncores' must be a positive integer.")
    }
  } 

  ###############################
  # Calculate Season

    # Correction need if monini is not January: 
    moninf <- moninf - monini + 1
    monsup <- monsup - monini + 1
    moninf <- ifelse(moninf <= 0, moninf + 12, moninf)
    monsup <- ifelse(monsup <= 0, monsup + 12, monsup)

    while (monsup < moninf) {
        monsup <- monsup + 12
    }

#    res <- Apply(list(data), 
#                 target_dims = time_dim, 
#                 output_dims = time_dim,
#                 fun = .Season, 
#                 monini = monini, moninf = moninf, monsup = monsup,
#                 method = method, na.rm = na.rm, ncores = ncores)$output1
    res <- apply(data, seq(1, length(dim(data)))[-match(time_dim, names(dim(data)))], s2dv:::.Season, 
                 monini = monini, moninf = moninf, monsup = monsup,
                 method = method, na.rm = na.rm)
    return(res)
}

#__________________Test 3 ________________________
benchmark(
          "s2dverification::Season" = {
            res_Regression <- s2dverification::Season(data, monini = 11, moninf = 12,
                                                      monsup = 1)
          },
          "s2dv::Season" = {
            res_Season <- s2dv::Season(data, monini = 11, moninf = 12, monsup = 1)
          },
          "s2dv::Season_apply" = {
            res_Season <- Season_apply(data, monini = 11, moninf = 12, monsup = 1)
          },

          replications = 10,
          columns = c("test", "replications", "elapsed",
                      "relative", "user.self", "sys.self"))

#                     test replications elapsed relative user.self sys.self
#2            s2dv::Season           10 108.201  644.054   108.196        0
#3      s2dv::Season_apply           10  42.411  252.446    42.408        0
#1 s2dverification::Season           10   0.168    1.000     0.167        0

# Season_apply is faster than Apply() but slower than s2dverification::Season.



