Trend <- function(data, sdate_dim = 'sdate', interval = 1, siglev = 0.95, conf = TRUE, polydeg = 1) {
  
  # Check inputs 
  ## data
  if (is.null(data)) {
    stop("Parameter 'data' cannot be NULL.")
  }
  if (!is.numeric(data)) {
    stop("Parameter 'data' must be a numeric array.")
  }
  if (is.null(dim(data))) {  #is vector
    dim(data) <- c(length(data))
    names(dim(data)) <- sdate_dim
  }
  if(any(is.null(names(dim(data))))| any(nchar(names(dim(data))) == 0)) {
    stop("Parameter 'data' must have dimension names.")
  }
  ## sdate_dim
  if (!is.character(sdate_dim) | length(sdate_dim) > 1) {
    stop("Parameter 'sdate_dim' must be a character string.")
  }
  if (!sdate_dim %in% names(dim(data))) {
    stop("Parameter 'sdate_dim' is not found in 'data' dimension.")
  }
  ## interval
  if (any(!is.numeric(interval) | interval <= 0 | length(interval) > 1)) {
    stop("Parameter 'interval' must be a positive number.")
  }
  ## siglev
  if (!is.numeric(siglev) | siglev < 0 | siglev > 1 | length(siglev) > 1) {
    stop("Parameter 'siglev' must be a numeric number between 0 and 1.")
  }
  ## conf
  if (!is.logical(conf) | length(conf) > 1) {
    stop("Parameter 'conf' must be one logical value.")
  }
  ## polydeg
  if (!is.numeric(polydeg) | polydeg %% 1 != 0 | polydeg < 0 | 
      length(polydeg) > 1) {
    stop("Parameter 'polydeg' must be a positive integer.")
  }
  
  ###############################
  # Calculate Trend
  dim_names <- names(dim(data))
  
  output <- Apply(list(data), 
                  target_dims = sdate_dim, 
                  fun = .Trend,
                  #output_dims = ,
                  interval = interval, siglev = siglev, conf = conf,
                  polydeg = polydeg)
  
  #str(output)
  #List of 4
  # $ trend     : num [1:2, 1, 1, 1:4, 1:57, 1:150] 1.02e+05 -2.80e+01 1.02e+05 4.24e-01 1.02e+05 ...
  # $ conf.lower: num [1, 1, 1:4, 1:57, 1:150] -189.4 -135.3 -104.4 -59.4 -183.6 ...
  # $ conf.upper: num [1, 1, 1:4, 1:57, 1:150] 133 136 197 167 132 ...
  # $ detrended : num [1:10, 1, 1, 1:4, 1:57, 1:150] 84 212 -30 -162 -904 ...
  
  reorder <- function(output) {
    
    # Add dim name back
    if (is.null(dim(output))) {
      dim(output) <- c(stats = length(output))
      
    } else {  #is an array
      if (length(dim(output)) == 1) {
        if (!is.null(names(dim(output)))) { 
          dim(output) <- c(1, dim(output))
          names(dim(output))[1] <- sdate_dim
        } else {
          names(dim(output)) <- sdate_dim
        }
      } else {  # more than one dim
        if (names(dim(output))[1] != "") {
          dim(output) <- c(1, dim(output))
          names(dim(output))[1] <- sdate_dim
        } else {   #regular case
          names(dim(output))[1] <- sdate_dim
        }
      }
    }
    # reorder
    pos <- match(dim_names, names(dim(output)))
    output <- aperm(output, pos)
    names(dim(output)) <- dim_names
    names(dim(output))[names(dim(output)) == sdate_dim] <- 'stats'

    return(output)
  }
  output <- lapply(output, reorder)

  return(output)
}

.Trend <- function(x, interval = interval, siglev = siglev, conf = conf, polydeg = polydeg) {
  
  #  # Checks
  #    if (!is.numeric(x)) {
  #        stop("Parameter 'x' must be a numeric vector.")
  #    }
  #    if (!is.numeric(interval) | interval %% 1 != 0 | interval < 0) {
  #        stop("Parameter 'interval' must be a positive integer.")
  #    } 
  #    if (length(interval) > 1) {
  #        interval <- interval[1]
  #        warning("Parameter 'interval' has length > 1 and only the first ",
  #                "element will be used.")
  #    }
  #    if (!is.numeric(siglev) | siglev < 0 | siglev > 1) {
  #        stop("Parameter 'siglev' must be a numeric number between 0 and 1.")
  #    }
  #    if (length(siglev) > 1) {
  #        siglev <- siglev[1]
  #        warning("Parameter 'siglev' has length > 1 and only the first ",
  #                "element will be used.")
  #    }
  #
  #    if (!is.logical(conf)) {
  #        stop("Parameter 'conf' must be logical.")
  #    }
  #    if (length(conf) > 1) {
  #        conf <- conf[1]
  #        warning("Parameter 'conf' has length > 1 and only the first ",
  #                "element will be used.")
  #    }
  #    if (!is.numeric(pldegree) | pldegree %% 1 != 0 | pldegree < 0) {
  #        stop("Parameter 'pldegree' must be a positive integer.")
  #    }
  #    if (length(pldegree) > 1) {
  #        pldegree <- pldegree[1]
  #        warning("Parameter 'pldegree' has length > 1 and only the first ",
  #                "element will be used.")
  #    }
  
  # Calculate trend
  if (any(!is.na(x))) {
    mon <- seq(x) * interval
    lm.out <- lm(x ~ poly(mon, degree = polydeg, raw = TRUE), na.action = na.omit)
    trend <- lm.out$coefficients  #intercept, slope1, slope2,...
    
    if (conf) {
      conf.lower <- confint(lm.out, level = siglev)[, 1]
      conf.upper <- confint(lm.out, level = siglev)[, 2]
    }
    
    detrended <- c()
    detrended[is.na(x) == FALSE] <- x[is.na(x) == FALSE] - lm.out$fitted.values
  } else {
    trend <- NA
    detrend <- NA
    if (conf) {
      conf.lower <- NA
      conf.upper <- NA
    }
  }
  
  if (conf) {
    return(list(trend = trend, conf.lower = conf.lower, conf.upper = conf.upper, 
                detrended = detrended))
  } else {
    return(list(trend = trend, detrended = detrended))
  }
  
}

