library(startR)
library(compiler)
library(multiApply)

# NOTE: This script is tested in the following environment:
#       R/3.6.2; Nord3 interactive session; 
#       multiApply_2.1.1; startR_2.0.3

#------------------------ Case 1 ----------------------
# A simple script calculating correlation between two arrays.
# Use erainterim daily mean reanalysis data.
#-------------------------------------------------------

#---------- Step 1: Load the data ---------------
# tos
  repos_tos <- paste0('/esarchive/recon/ecmwf/erainterim/daily_mean/',
                      '$var$_f24h/$var$_$sdate$.nc')

# full period: 197901-201807
#  sdate <- sapply(1979:2017, function(x) paste0(x, sprintf('%02d', 1:12))) #longer period
  sdate <- sapply(1998:2017, function(x) paste0(x, sprintf('%02d', 7)))

  tos <- Start(dat = repos_tos,
               var = 'tos',
               sdate = sdate,
               time = 'all',
               time_across = 'sdate',
               merge_across_dims = TRUE,
               latitude = 'all',
               longitude = 'all',
               return_vars = list(latitude = NULL,
                                  longitude = NULL,
                                  time = 'sdate'),
               retrieve = T)
  tos <- ClimProjDiags::Subset(tos, c(1, 2), list(1, 1), drop = 'selected')

# prc
  repos_prc <- paste0('/esarchive/recon/ecmwf/erainterim/daily_mean/',
                      '$var$_s12-24h/$var$_$sdate$.nc')

  prc <- Start(dat = repos_prc,
               var = 'prc',
               sdate = sdate,
               time = 'all',
               time_across = 'sdate',
               merge_across_dims = TRUE,
               latitude = 'all',
               longitude = 'all',
               return_vars = list(latitude = NULL,
                                  longitude = NULL,
                                  time = 'sdate'),
               retrieve = T)
  prc <- ClimProjDiags::Subset(prc, c(1, 2), list(1, 1), drop = 'selected')


#----------- Step 2: Define functions --------------
# (1) correlation function (for loop)
  func_cor_forloop <- function (x, y) {
    len_lat <- dim(x)[2]
    len_lon <- dim(x)[3]
    res <- array(dim = c(dim(x)[2], dim(x)[3]))
    for (i in 1:len_lat) {
      for (j in 1:len_lon) {
        res[i, j] <- cor(x[, i, j], y[, i, j])
      }
    }
 
    return(res)
  }

# (2) correlation function (multiApply)
  func_cor_Apply <- function (x, y) {
    .cor <- function (xx, yy) {
      # input is 1-dim array
      cor(as.vector(xx), as.vector(yy))
    }

    res <- Apply(list(x, y), 
                 target_dims = list(c('time'), 
                                    c('time')),
                 fun = .cor)$output1
    return(res)
  }

# (3) compiled function 1
  compiled_func_cor_forloop <- cmpfun(func_cor_forloop)

# (4) compiled function 2
  compiled_func_cor_Apply <- cmpfun(func_cor_Apply)
  

#------------ Step 3: Run the functions --------------
# disable JIT first
enableJIT(0)

# (1)
  start_time <- Sys.time()
  res_cor_forloop <- func_cor_forloop(tos, prc)
  end_time <- Sys.time()
  total_time_forloop <- end_time - start_time
#-> Time difference of 8.403877 secs

# (2)
  start_time <- Sys.time()
  res_cor_Apply <- func_cor_Apply(tos, prc)
  end_time <- Sys.time()
  total_time_Apply <- end_time - start_time
#-> Time difference of 19.81281 secs

# (3)
  start_time <- Sys.time()
  res_cor_forloop_compiled <- compiled_func_cor_forloop(tos, prc)
  end_time <- Sys.time()
  total_time_forloop_comp <- end_time - start_time
#-> Time difference of 7.728763 secs

# (4)
  start_time <- Sys.time()
  res_cor_Apply_compiled <- compiled_func_cor_forloop(tos, prc)
  end_time <- Sys.time()
  total_time_Apply_comp <- end_time - start_time
#-> Time difference of 7.801257 secs


