# Load startR
library(startR)

# Define a region
lons.min <- 0
lons.max <- 360
lats.min <- -90
lats.max <- 90

# Declaration of data sources
exp <- Start(dat = '/esarchive/exp/ecmwf/system4_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'tas',
             sdate = paste0(1981:2010, '0101'),
             ensemble = 'all',
             time = indices(1),
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(),
             longitude = values(list(lons.min, lons.max)),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = 'r360x181',
                                     method = 'conservative',
                                     crop = c(lons.min, lons.max, lats.min, lats.max)),
             transform_vars = c('latitude', 'longitude'),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c('sdate')),
             retrieve = FALSE)

obs <- Start(dat = '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'tas',
             sdate = paste0(1981:2010, '01'),
             time = indices(1),
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(),
             longitude = values(list(lons.min, lons.max)),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = 'r360x181',
                                     method = 'conservative',
                                     crop = c(lons.min, lons.max, lats.min, lats.max)),
             transform_vars = c('latitude', 'longitude'),
             synonims = list(latitude = c('latitude', 'lat'),
                             longitude = c('longitude', 'lon')),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c('sdate')),
             retrieve = FALSE)

# Define the workflow
## Function
wrap_cal <- function(obs, exp) {
    obs <- s2dv::InsertDim(obs, 1, 1, name = 'member')
    names(dim(obs))<- c('member', 'sdate')
    exp <- t(exp)
    names(dim(exp))<- c('member', 'sdate')
    calibrated <- CSTools:::.cal(var_obs = obs, var_exp = exp)

    #ensemble mean
    asd <- apply(calibrated, c(which(names(dim(calibrated)) != 'member')), mean)
    #dim(asd) <- c(sdate = length(asd)) 
    #climatology
    qwe <- mean(asd)
    dim(qwe) <- c(sdate = 1)
    return(qwe)
}

## Workflow
step <- Step(wrap_cal,
             target_dims = list(obs = c('sdate'), exp = c('sdate', 'ensemble')),
             output_dims = c('sdate'))

wf <- AddStep(list(obs = obs, exp = exp), step)


# Compute() locally
res1 <- Compute(wf, chunks = list(latitude = 2, longitude = 2),
                threads_load = 2, threads_compute = 4)

# Compute() on fatnodes
  res <- Compute(wf,
                 chunks = list(latitude = 1,
                               longitude = 1),#$output1
                 threads_load = 2,
                 threads_compute = 4,
                 cluster = list(queue_host = 'fat',
                                queue_type = 'slurm',
                                cores_per_job = 2,
                                job_wallclock = '01:30:00',
                                max_jobs = 4,
                                bidirectional = TRUE
                 ),
                 ecflow_suite_dir = '/home/Earth/aho/startR_local/'
                 wait = TRUE)

# Compute() on Power 9
  res1 <- Compute(wf,
                 chunks = list(latitude = 1,
                               longitude = 1),#$output1
                 threads_load = 2,
                 threads_compute = 4,
                 cluster = list(queue_host = 'p1',
                                queue_type = 'slurm',
                                temp_dir = '/gpfs/scratch/bsc32/bsc32734/startR_hpc/',
                                r_module = 'R/3.5.0-foss-2018b',
                                CDO_module = 'CDO/1.9.5-foss-2018b',
                                job_wallclock = '01:00:00',
                                cores_per_job = 4,
                                max_jobs = 4,
                                bidirectional = FALSE,
                                polling_period = 50
                               ),
                 ecflow_suite_dir = '/home/Earth/aho/startR_local/',
                 wait = TRUE
                 )  #$output1

#Visualize
PlotEquiMap(res1$output1[1,1,1,1,,],c(1:360),c(-90:90), filled.continents = FALSE, 
            toptitle = 'tas ensemble climatology after calibration (Jan 1981-2010)', title_scale = 0.6)


