# Load startR
library(startR)

# Define a region
lons.min <- 0
lons.max <- 359.99
lats.min <- -90
lats.max <- 90
year <- 1981:2010
sdates <- array(paste0(rep(year, each = 12), sprintf("%02d", 1:12), '01'),
                dim = c(smonth = 12, syear = length(year)))

# Declaration of data sources
exp <- Start(dat = '/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'tas',
             sdate = sdates,
             split_multiselected_dims = TRUE,
             ensemble = 'all',
             time = indices(1),
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(),
             longitude = values(list(lons.min, lons.max)),
             longitude_reorder = CircularSort(0, 360),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = 'r360x181',
                                     method = 'conservative',
                                     crop = c(lons.min, lons.max, lats.min, lats.max)),
             transform_vars = c('latitude', 'longitude'),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c('sdate')),
             retrieve = FALSE)

sdates_obs <- array(paste0(rep(year, each = 12), sprintf("%02d", 1:12)),
                    dim = c(smonth = 12, syear = length(year)))
obs <- Start(dat = '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_f6h/$var$_$sdate$.nc',
             var = 'tas',
             sdate = sdates_obs,
             split_multiselected_dims = TRUE,
             time = indices(1),
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(),
             longitude = values(list(lons.min, lons.max)),
             longitude_reorder = CircularSort(0, 360),
             transform = CDORemapper,
             transform_extra_cells = 2,
             transform_params = list(grid = 'r360x181',
                                     method = 'conservative',
                                     crop = c(lons.min, lons.max, lats.min, lats.max)),
             transform_vars = c('latitude', 'longitude'),
             synonims = list(latitude = c('latitude', 'lat'),
                             longitude = c('longitude', 'lon')),
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = c('sdate')),
             retrieve = FALSE)

# Define the workflow
## Function
wrap_cal <- function(obs, exp) {
    calibrated <- CSTools:::.cal(exp = exp, obs = obs, 
                                 cal.method = "bias", eval.method = "leave-one-out",
                                 multi.model = FALSE, na.fill = TRUE, na.rm = TRUE, 
                                 apply_to = NULL, alpha = NULL)  # [ensemble, syear]
    #ensemble mean
    ens_mean <- apply(calibrated, 2, mean, na.rm = TRUE)
    #climatology
    clim <- mean(ens_mean, na.rm = TRUE)
    return(clim)
}

## Workflow
step <- Step(wrap_cal,
             target_dims = list(obs = c('syear'), exp = c('ensemble', 'syear')),
             output_dims = NULL)

wf <- AddStep(list(obs = obs, exp = exp), step)


# Compute() locally
res1 <- Compute(wf, chunks = list(smonth = 2),
                threads_load = 2, threads_compute = 4)


#-------------------user-defined---------------------
  queue_host <- 'nord1'
  temp_dir <- '/gpfs/scratch/bsc32/bsc32734/startR_hpc/'
  ecflow_suite_dir <- '/home/Earth/aho/startR_local/'
#----------------------------------------------------

  res <- Compute(wf,
                 chunks = list(smonth = 2),
                 threads_load = 2,
                 threads_compute = 4,
                 cluster = list(queue_host = queue_host,
                   queue_type = 'lsf',
                   temp_dir = temp_dir,
                   cores_per_job = 2,
                   job_wallclock = '05:00',
                   max_jobs = 4,
                   extra_queue_params = list('#BSUB -q bsc_es'),
                   bidirectional = FALSE,
                   polling_period = 10
                 ),
                 ecflow_suite_dir = ecflow_suite_dir,
                 wait = TRUE
                 )  #$output1


#Visualize
PlotEquiMap(res1$output1[1,1,1,1,,],c(1:360),c(-90:90), filled.continents = FALSE,
            toptitle = 'tas ensemble climatology after calibration (Jan 1981-2010)', title_scale = 0.6)

