```r
library(startR)
```

A) Load mask file
```r
mask_path <- '/esarchive/autosubmit/con_files/land_sea_mask_1x1_c3s.nc'

mask <- Start(dat = mask_path,
              var =  'LSM',
              lat = 'all',
              lon = 'all',
              return_vars = list(var_names = NULL,
                                 lat = NULL, lon = NULL),
              var_var = 'var_names',
              retrieve = F)
```

B) Load tos and tas data
```r
data_path <- '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$year$$month$01.nc'
var <- c('tas', 'tos')
year <- as.character(1991:2010)
month <- sprintf("%02d", 1:12)

data_tas <- Start(dat = data_path,
                  var = var[1],
                  year = year,
                  month = month,
                  ensemble = 'all',
                  time = indices(1),
                  lat = 'all',
                  lon = 'all',
                  synonims = list(lat = c('lat', 'latitude'),
                                  lon = c('lon', 'longitude')),
                  return_vars = list(lat = NULL,
                                     lon = NULL,
                                     time = c('year', 'month')),
                  retrieve = FALSE)

data_tos <- Start(dat = data_path,
                  var = var[2],
                  year = year,
                  month = month,
                  ensemble = 'all',
                  time = indices(1),
                  lat = 'all',
                  lon = 'all',
                  synonims = list(lat = c('lat', 'latitude'),
                                  lon = c('lon', 'longitude')),
                  return_vars = list(lat = NULL,
                                     lon = NULL,
                                     time = c('year', 'month')),
                  retrieve = FALSE)
```

C) Build the workflow

```r
reg_mask <- function(data1, data2, mask) {
  # datas: [dat = 1, year = 20, month = 12, ensemble = 25, time = 1]
  # mask: [dat = 1]

  if (mask == 1) {

  # Remove the unecessary dims
#   data1 <- ClimProjDiags::Subset(data1, c('dat', 'time'), list(1, 1), drop = 'selected')
#   data2 <- ClimProjDiags::Subset(data2, c('dat', 'time'), list(1, 1), drop = 'selected')
    mth_len <- as.numeric(dim(data1)[3])
    nexp_len <- as.numeric(dim(data1)[4])
    nobs_len <- as.numeric(dim(data2)[4])
    data1 <- data1[1, , , , 1]
    data2 <- data2[1, , , , 1]
    corr <- array(dim = c(nexp = nexp_len, nobs = nobs_len, month = mth_len))

    for (mth in 1:mth_len) {
      # Input of .Corr(): [year = 20, ensemble = 25]
      corr[, , mth] <- s2dv:::.Corr(data1[, mth, ], data2[, mth, ], time_dim = 'year',
                                    pval = FALSE, conf = FALSE)$corr
    }
      # corr: [nexp = 25, nobs = 25, month = 12]
      # ensemble mean
      ens_mean <- s2dv::MeanDims(corr, c('nexp', 'nobs'))
      # ens_mean: [month = 12]

  } else {
    ens_mean <- array(rep(NA, dim(data1)[3]), dim = c(month = dim(data1)[3]))
  }
  return(ens_mean)
}

step_mask <- Step(fun = reg_mask,
                  target_dims = list(data1 = c('dat', 'year', 'month', 'ensemble', 'time'),
                                     data2 = c('dat', 'year', 'month', 'ensemble', 'time'),
                                     mask = c('dat')),
                  output_dim = 'month')

wf_mask <- AddStep(list(data1 = data_tas, data2 = data_tos, mask = mask), step_mask)

```

```r
# run locally
res <- Compute(workflow = wf_mask,
                chunks = list(lat = 2,
                              lon = 2))
```


```r
# Submit to Nord3
# NOTE: Ensure your local R version is >= 3.5.0!!!
#-------------------user-defined---------------------
  queue_host <- 'nord1'
  temp_dir <- '/gpfs/scratch/bsc32/bsc32734/startR_hpc/'
  ecflow_suite_dir <- '/home/Earth/aho/startR_local/'
#----------------------------------------------------
  res <- Compute(wf_mask,
                 chunks = list(lat = 2,
                               lon = 2),#$output1
                 threads_load = 1,
                 threads_compute = 4,
                 cluster = list(queue_host = queue_host,
                                queue_type = 'lsf',
                                temp_dir = temp_dir,
                                cores_per_job = 2,
                                job_wallclock = '05:00',
                                max_jobs = 4,
                                extra_queue_params = list('#BSUB -q bsc_es'),
                                bidirectional = FALSE,
                                polling_period = 10
                 ),
                 ecflow_suite_dir = ecflow_suite_dir,
                 wait = FALSE
                 )

# Save the header of res
save_path <- '/home/Earth/aho/aho-testtest/startR/tutorial/'
save_file <- paste0(save_path, 'hands-on_Compute.Rds')
saveRDS(res, file = save_file)

# You can leave now. When you come back next time, just read the .Rds file 
res_collect <- readRDS(save_file)
result <- Collect(res_collect, wait = TRUE)

saveRDS(result, file = paste0(save_path, 'hands-on-2.Rds'))

```

# Plot the result
```r
library(s2dv)
s2dv::PlotLayout(PlotEquiMap, c('lat', 'lon'), result$output1[ , 1, , ], 
                 lon = seq(0, 359, 1), lat = seq(90, -90, -1),
                 toptitle = "ECMWF system5c3s tas-tos Correlation 1991-2010",
                 titles = c('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
                            'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'),
                 bar_scale = 0.5, title_scale = 0.6) 
```


