#-------------------------------------------------------------------------
# Since startR hasn't been fully stable with many developments going on,
# it is important to self-check if the retrieving data array is as you expect.
# This script provided several tools and tips to ensure everything is fine
# before you head to further analysis.

# Other data retrieval tools: (1) s2dv::Load  (2) ncdf4  (3) easyNCDF  (4) ncview
# Tips: (a) dimensions  (b) attributes  (c) summary  (d) individual values
# Extra examination: (5) Compute()
#-------------------------------------------------------------------------

#========== (1) s2dv::Load ===========
# If you used Load() before, it is convenient to compare with its output directly.
# It is useful when transformation is applied. Other methods provided here
# can only show original data. Also, it helps check the retrieved data structure,
# not only the values.

# startR
library(startR)

obs_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h/$var$_$sdate$.nc'
var_name <- 'sfcWind'
sdate <- '201811'
lons.min <- 10
lons.max <- 20
lats.min <- 0
lats.max <- 10

obs1 <- Start(dat = obs_path,
              var = var_name,
              sdate = sdate,
              time = 'all',
              latitude = values(list(lats.min, lats.max)),
              latitude_reorder = Sort(decreasing = T),
              longitude = values(list(lons.min, lons.max)),
              longitude_reorder = CircularSort(0, 360),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              transform = CDORemapper,
              transform_extra_cells = 2,
              transform_params = list(grid = 'r360x181',
                                      method = 'conservative',
                                      crop = c(lons.min, lons.max, lats.min, lats.max)),
              transform_vars = c('latitude', 'longitude'),
              return_vars = list(time = NULL,
                                 latitude = 'dat',
                                 longitude = 'dat'),
              retrieve = T)

# Load()
library(s2dv)
pobs <- paste0('/esarchive/recon/ecmwf/era5/monthly_mean/',
               '$VAR_NAME$_f1h/$VAR_NAME$_$YEAR$$MONTH$.nc')

obs2 <- Load(var = var_name,
             obs = list(list(path = pobs)),
             sdates = '20181101',
             leadtimemin = 1,
             leadtimemax = 1,
             output = 'lonlat',
             grid = 'r360x181',
             method = 'conservative',
             storefreq = 'monthly',
             latmin = lats.min,
             latmax = lats.max,
             lonmin = lons.min,
             lonmax = lons.max)

# Check
## (a) dimensions
dim(obs1)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        11        11 
dim(obs2$obs)
#  dataset  member   sdate   ftime     lat     lon 
#        1       1       1       1      11      11 

## (b) attributes
attr(obs1, 'Variables')$dat1$longitude
# [1] 10 11 12 13 14 15 16 17 18 19 20
as.vector(obs2$lon)
# [1] 10 11 12 13 14 15 16 17 18 19 20
attr(obs1, 'Variables')$dat1$latitude
# [1] 10  9  8  7  6  5  4  3  2  1  0
as.vector(obs2$lat)
# [1] 10  9  8  7  6  5  4  3  2  1  0

## (c) summary
diff <- drop(obs1) - drop(obs2$obs)
range(diff)
#[1] -4.965553e-05  4.929314e-05
summary(obs1)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#  1.090   1.313   1.479   1.636   1.936   2.803 
summary(obs2$obs)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#  1.090   1.313   1.479   1.636   1.936   2.803 

## (d) individual values
obs1[1, 1, 1, 1, 1:3, 1:2]
#         [,1]     [,2]
#[1,] 2.345519 2.365326
#[2,] 2.219121 1.958319
#[3,] 1.840537 1.386617
obs2$obs[1, 1, 1, 1, 1:3, 1:2]
#       [,1]   [,2]
#[1,] 2.3455 2.3653
#[2,] 2.2191 1.9583
#[3,] 1.8405 1.3866


#========== (2) ncdf4 ===========
# It is resource-consumning if the whole domain is retrieved through ncdf4..
# If doing so, use fatnodes/Power9/Nord3 interactive session for more memory space.
# Check ncdf4 documentation to learn the dimension structure first.

# startR
library(startR)
obs_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h/$var$_$sdate$.nc'
var_name <- 'sfcWind'
sdate <- '201811'

obs1 <- Start(dat = obs_path,
              var = var_name,
              sdate = sdate,
              time = 'all',
              latitude = indices(1:10), 
              longitude = indices(1:10),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              return_vars = list(time = 'sdate',
                                 latitude = 'dat',
                                 longitude = 'dat'),
              retrieve = T)

# ncdf4
library(ncdf4)
file <- nc_open('/esarchive/recon/ecmwf/era5/monthly_mean/sfcWind_f1h/sfcWind_201811.nc')
obs2 <- ncvar_get(file, 'sfcWind', start = c(1, 1, 1), count = c(10, 10, 1))
nc_close(file)

# Check
## (a) dimensions
dim(obs1)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        10        10 
dim(obs2)
#[1] 10 10

## (c) summary
summary(obs1)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#  4.934   4.954   5.025   5.035   5.105   5.165 
summary(as.vector(obs2))
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#  4.934   4.954   5.025   5.035   5.105   5.165

## (d) individual values
obs1[1, 1, 1, 1, , 2]
# [1] 4.960041 4.959622 4.956322 4.939547 4.991117 5.075977 5.143179 5.157578
# [9] 5.095344 5.051348
obs2[2, ]
# [1] 4.960041 4.959622 4.956322 4.939547 4.991117 5.075977 5.143179 5.157578
# [9] 5.095344 5.051348


#========== (3) easyNCDF ===========
# startR
# Use the same one as (2).

# easyNCDF
library(easyNCDF)
file <- NcOpen('/esarchive/recon/ecmwf/era5/monthly_mean/sfcWind_f1h/sfcWind_201811.nc')

obs2 <- NcToArray(file, vars_to_read = 'sfcWind', 
                  dim_indices = list(longitude = c(1:10), 
                                     latitude = c(1:10),
                                     time = c(1)))
NcClose(file)

# Check
## (a) dimensions
dim(obs1)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        10        10 
dim(obs2)
#      var longitude  latitude      time 
#        1        10        10         1 

## (c) summary
summary(obs1)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#  4.934   4.954   5.025   5.035   5.105   5.165 
summary(as.vector(obs2))
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#  4.934   4.954   5.025   5.035   5.105   5.165 

## (d) individual values
obs1[1, 1, 1, 1, , 2]
# [1] 4.960041 4.959622 4.956322 4.939547 4.991117 5.075977 5.143179 5.157578
# [9] 5.095344 5.051348
obs2[1, 2, , 1]
# [1] 4.960041 4.959622 4.956322 4.939547 4.991117 5.075977 5.143179 5.157578
# [9] 5.095344 5.051348


#========== (4) ncview ===========
# In the terminal, type 'ncview <file_name>'. Then check the values dynamically in the pop-up window.
# You can also plot the map to compare with ncview. It is more clear if you use a bigger domain (rather than the example here, which only has 10*10 points)

s2dv::PlotEquiMap(obs1[1,1,1,1,,], 
                  lon = attributes(obs1)$Variable$dat1$longitude,
                  lat = attributes(obs1)$Variable$dat1$latitude)

#========== (5) Compute() ===========
# You can compare the result of Compute() with what you get from Start(retrieve = TRUE), or with any tool mentioned above.

# startR (retrieve = TRUE)
library(startR)

obs_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h/$var$_$sdate$.nc'
var_name <- 'sfcWind'
sdate <- '201811'
lons.min <- 10
lons.max <- 20
lats.min <- 0
lats.max <- 10

obs1 <- Start(dat = obs_path,
              var = var_name,
              sdate = sdate,
              time = 'all',
              latitude = values(list(lats.min, lats.max)),
              latitude_reorder = Sort(decreasing = T),
              longitude = values(list(lons.min, lons.max)),
              longitude_reorder = CircularSort(0, 360),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              transform = CDORemapper,
              transform_extra_cells = 2,
              transform_params = list(grid = 'r360x181',
                                      method = 'conservative',
                                      crop = c(lons.min, lons.max, lats.min, lats.max)),
              transform_vars = c('latitude', 'longitude'),
              return_vars = list(time = NULL,
                                 latitude = 'dat',
                                 longitude = 'dat'),
              retrieve = T)


# startR (retrieve = FALSE, use the whole workflow)

obs2 <- Start(dat = obs_path,
              var = var_name,
              sdate = sdate,
              time = 'all',
              latitude = values(list(lats.min, lats.max)),
              latitude_reorder = Sort(decreasing = T),
              longitude = values(list(lons.min, lons.max)),
              longitude_reorder = CircularSort(0, 360),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              transform = CDORemapper,
              transform_extra_cells = 2,
              transform_params = list(grid = 'r360x181',
                                      method = 'conservative',
                                      crop = c(lons.min, lons.max, lats.min, lats.max)),
              transform_vars = c('latitude', 'longitude'),
              return_vars = list(time = NULL,
                                 latitude = 'dat',
                                 longitude = 'dat'),
              retrieve = F)

func <- function(x) {
  return(x)
}

step <- Step(func,
             target_dims = 'latitude',
             output_dims = 'latitude')

wf <- AddStep(obs2, step)

res <- Compute(wf, chunks = list(time = 1))$output1

# Check
## (a) dimensions
dim(obs1)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        11        11 
dim(res)
# latitude       dat       var     sdate      time longitude 
#       11         1         1         1         1        11 

## (c) summary
diff <- drop(obs1) - drop(res)
range(diff)
#[1] 0 0
summary(obs1)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#  1.090   1.313   1.479   1.636   1.936   2.803 
summary(res)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#  1.090   1.313   1.479   1.636   1.936   2.803 

## (d) individual values
obs1[1, 1, 1, 1, 1:3, 1:2]
#         [,1]     [,2]
#[1,] 2.345519 2.365326
#[2,] 2.219121 1.958319
#[3,] 1.840537 1.386617
res[1:3, 1, 1, 1, 1, 1:2]
#         [,1]     [,2]
#[1,] 2.345519 2.365326
#[2,] 2.219121 1.958319
#[3,] 1.840537 1.386617


